@@ -1,4 +1,3 @@
-
 =head1 NAME
 
 Parse::BooleanLogic - parser of boolean expressions
@@ -27,9 +26,51 @@ Request Tracker for parsing SQL like expressions and it's still capable to, but
 it can be used to parse other boolean logic sentences with OPERANDs joined using
 binary OPERATORs and grouped and nested using parentheses (OPEN_PAREN and CLOSE_PAREN).
 
+=cut
+
+
+
+use strict;
+use warnings;
+
+package Parse::BooleanLogic;
+
+our $VERSION = '0.03';
+
+use constant OPERAND     => 1;
+use constant OPERATOR    => 2;
+use constant OPEN_PAREN  => 4;
+use constant CLOSE_PAREN => 8;
+my @tokens = qw[OPERAND OPERATOR OPEN_PAREN CLOSE_PAREN];
+
+use Regexp::Common qw(delimited);
+my $re_operator    = qr[(?i:AND|OR)];
+my $re_open_paren  = qr[\(];
+my $re_close_paren = qr[\)];
+
+my $re_tokens      = qr{(?:$re_operator|$re_open_paren|$re_close_paren)};
+my $re_delim       = qr{$RE{delimited}{-delim=>qq{\'\"}}};
+my $re_operand     = qr{(?!\s)(?:$re_delim|(?!$re_tokens|["']).+?(?=$re_tokens|["']|\Z))+};
+
 =head1 METHODS
 
-=head2 as_array
+=head2 new
+
+Very simple constructor, returns a new object. Now takes no options and most
+methods can be executed as class methods too, however there are plans to
+change it and using this lightweight constructor is recommended.
+
+=cut
+
+sub new {
+    my $proto = shift;
+    return bless {@_}, ref($proto) || $proto;
+}
+
+
+=head2 Parsing expressions
+
+=head3 as_array $string [ %options ]
 
 Takes a string and parses it into perl structure, where parentheses represented using
 array references, operands are hash references with one key/value pair: operand,
@@ -46,7 +87,53 @@ is parsed into the following structure:
         ]
     ]
 
-=head2 parse
+Aditional options:
+
+=over 4
+
+=item operand_cb - custom operands handler
+
+=item error_cb - custom errors handler
+
+=back
+
+=cut
+
+{ # static variables
+
+my ($tree, $node, @pnodes);
+my %callback;
+$callback{'open_paren'} = sub {
+    push @pnodes, $node;
+    push @{ $pnodes[-1] }, $node = []
+};
+$callback{'close_paren'}     = sub { $node = pop @pnodes };
+$callback{'operator'} = sub { push @$node, $_[0] };
+$callback{'operand'} = sub { push @$node, { operand => $_[0] } };
+
+sub as_array {
+    my $self = shift;
+    my $string = shift;
+    my %arg = (@_);
+
+    $node = $tree = [];
+    @pnodes = ();
+
+    unless ( $arg{'operand_cb'} || $arg{'error_cb'} ) {
+        $self->parse(string => $string, callback => \%callback);
+        return $tree;
+    }
+
+    my %cb = %callback;
+    if ( $arg{'operand_cb'} ) {
+        $cb{'operand'} = sub { push @$node, $arg{'operand_cb'}->( $_[0] ) };
+    }
+    $cb{'error'} = $arg{'error_cb'} if $arg{'error_cb'};
+    $self->parse(string => $string, callback => \%cb);
+    return $tree;
+} }
+
+=head3 parse
 
 Takes named arguments: string and callback. Where the first one is scalar with
 expression, the latter is a reference to hash with callbacks: open_paren, operator
@@ -74,11 +161,11 @@ Here is simple example based on L</as_array> method:
 
     # on close_paren just switch to previous group by taking it
     # from the top of the stack
-    $callback{'close_paren'}     = sub { $node = pop @pnodes };
+    $callback{'close_paren'} = sub { $node = pop @pnodes };
 
     # push binary operators as is and operands as hash references
     $callback{'operator'} = sub { push @$node, $_[0] };
-    $callback{'operand'}       = sub { push @$node, { operand => $_[0] } };
+    $callback{'operand'}  = sub { push @$node, { operand => $_[0] } };
 
     # run parser
     $parser->parse( string => $string, callback => \%callback );
@@ -89,57 +176,6 @@ Using this method you can build other representations of an expression.
 
 =cut
 
-use strict;
-use warnings;
-
-package Parse::BooleanLogic;
-
-our $VERSION = '0.02';
-
-use constant OPERAND     => 1;
-use constant OPERATOR    => 2;
-use constant OPEN_PAREN  => 4;
-use constant CLOSE_PAREN => 8;
-my @tokens = qw[OPERAND OPERATOR OPEN_PAREN CLOSE_PAREN];
-
-use Regexp::Common qw(delimited);
-my $re_operator    = qr[(?i:AND|OR)];
-my $re_open_paren  = qr[\(];
-my $re_close_paren = qr[\)];
-
-my $re_tokens      = qr{(?:$re_operator|$re_open_paren|$re_close_paren)};
-my $re_delim       = qr{$RE{delimited}{-delim=>qq{\'\"}}};
-my $re_operand     = qr{(?!\s)(?:$re_delim|(?!$re_tokens|["']).+?(?=$re_tokens|["']|\Z))+};
-
-sub new {
-    my $proto = shift;
-    return bless {}, ref($proto) || $proto;
-}
-
-{ # static variables
-
-my ($tree, $node, @pnodes);
-my %callback;
-$callback{'open_paren'} = sub {
-    push @pnodes, $node;
-    push @{ $pnodes[-1] }, $node = []
-};
-$callback{'close_paren'}     = sub { $node = pop @pnodes };
-$callback{'operator'} = sub { push @$node, $_[0] };
-$callback{'operand'} = sub { push @$node, { operand => $_[0] } };
-
-sub as_array {
-    my $self = shift;
-    my $string = shift;
-
-    $node = $tree = [];
-    @pnodes = ();
-
-    $self->parse(string => $string, callback => \%callback);
-
-    return $tree;
-} }
-
 sub parse {
     my $self = shift;
     my %args = (
@@ -236,6 +272,84 @@ sub bitmask_to_string {
     return join ' or ', @res;
 }
 
+=head2 Tree modifications
+
+Several functions taking a tree of boolean expressions as returned by
+as_array method and changing it using a callback.
+
+=head3 filter $tree $callback
+
+Returns sub-tree where only operands left for which the callback returned
+true value.
+
+=cut
+
+sub filter {
+    my ($self, $tree, $cb, $inner) = @_;
+
+    my $skip_next = 0;
+
+    my @res;
+    foreach my $entry ( @$tree ) {
+        next if $skip_next-- > 0;
+
+        if ( ref $entry eq 'ARRAY' ) {
+            my $tmp = $self->filter( $entry, $cb, 1 );
+            if ( !$tmp || (ref $tmp eq 'ARRAY' && !@$tmp) ) {
+                pop @res;
+                $skip_next = 1 unless @res;
+            } else {
+                push @res, $tmp;
+            }
+        } elsif ( ref $entry eq 'HASH' ) {
+            if ( $cb->( $entry ) ) {
+                push @res, $entry;
+            } else {
+                pop @res;
+                $skip_next = 1 unless @res;
+            }
+        } else {
+            push @res, $entry;
+        }
+    }
+    return $res[0] if @res == 1 && ($inner || ref $res[0] eq 'ARRAY');
+    return \@res;
+}
+
+=head3 solve $tree $callback
+
+Returns sub-tree where only operands left for which the callback returned
+true value.
+
+=cut
+
+sub solve {
+    my ($self, $tree, $cb) = @_;
+
+    my ($res, $ea, $skip_next) = (0, 'OR', 0);
+    foreach my $entry ( @$tree ) {
+        next if $skip_next-- > 0;
+        unless ( ref $entry ) {
+            $ea = $entry;
+            $skip_next++ if ($res && $ea eq 'OR') || (!$res && $ea eq 'AND');
+            next;
+        }
+
+        my $cur;
+        if ( ref $entry eq 'ARRAY' ) {
+            $cur = $self->solve( $entry, $cb, 1 );
+        } else {
+            $cur = $cb->( $entry );
+        }
+        if ( $ea eq 'OR' ) {
+            $res ||= $cur;
+        } else {
+            $res &&= $cur;
+        }
+    }
+    return $res? 1 : 0;
+}
+
 1;
 
 =head1 AUTHORS
